@@ -77,3 +77,9 @@ void Client::SendRegisterResult(ERegisterRes result){
 	this->SendNetworkData_ts(p);
 }
 
+
+
+void Client::SetState(EState newState){
+	this->state = newState;
+}
+
