@@ -73,19 +73,9 @@ void Therver::OnDataReceivedFromClient(ting::Ref<cliser::Client>& client, ting::
 
 //override
 void ClientDisconnectedMessage::Handle(){
-	this->c->SetState(Client::DISCONNECTED);
+	this->c->SetNetworkState(Client::DISCONNECTED);
 
-	//remove the client from cast it listening to, if it does
-	if(this->c->isListening){
-		(*this->c->listeningCast).RemoveListener(this->c);
-		this->c->isListening = false;
-	}
-
-	//if client has a playing cast then notify listeners that the cast has finished
-	//and remove the cast
-	if(this->c->isPlaying){
-		this->t->RemoveCast(this->c->playingCast);
-	}
+	this->t->MoveClientToIdleState(this->c);
 }
 
 
@@ -117,11 +107,13 @@ void NetworkDataFromClientReceivedMessage::Handle(){
 
 
 
-void Therver::HandleRegisterFromClient(ting::Ref< ::Client>& c, ting::Array<ting::byte> d){
+void Therver::HandleRegisterFromClient(ting::Ref<Client>& c, ting::Array<ting::byte> d){
 	ASSERT(c)
 	ASSERT(d.Size() > 0)
 	ASSERT(d[0] == 0x04) //4 = register command
 
+	//TODO: ignore if client is logged in
+
 	ting::uint i = 1;
 
 	if(i + 2 > d.Size()){
@@ -185,6 +177,8 @@ void Therver::HandleLoginFromClient(ting::Ref< ::Client>& c, ting::Array<ting::b
 	ASSERT(p.Size() > 0)
 	ASSERT(p[0] == 0)// 0 = login command
 
+	//TODO: ignore if already logged in
+
 	//vars to be filled
 	std::string un;
 	std::string pw;
@@ -250,7 +244,7 @@ void Therver::HandleLoginFromClient(ting::Ref< ::Client>& c, ting::Array<ting::b
 	c->username = un;
 	c->password = pw;
 
-	c->SetState(Client::LOGGING_IN);
+	c->SetNetworkState(Client::LOGGING_IN);
 
 	//send query request to DB thread
 	this->dbThread.PushMessage(Ptr<Message>(
@@ -274,20 +268,25 @@ void Therver::HandlePlayingDataFromClient(ting::Ref< ::Client>& c, ting::Array<t
 	//if the client does not play and does not have its own cast
 	//then create one, because we received playing data from player
 	//which means it started playing
-	if(!c->isPlaying){
+	if(!c->IsPlaying()){
+		this->MoveClientToIdleState(c);
+
 		//create new cast
 		this->casts.push_back(Cast(c));
 
 		ASSERT(this->casts.size() > 0)
 		T_CastsIter castIter = (--this->casts.end());
 
-		c->isPlaying = true;
-		c->playingCast = castIter;
+		c->SetState(Client::PLAYING);
+		c->cast = castIter;
 	}
 
-	ASSERT(c->isPlaying)
+	ASSERT(c->IsPlaying())
 
-	c->playingCast->PushNewMusicDataPacket(d);
+	c->cast->PushNewMusicDataPacket(d);
+
+	//if there are waiting listeners, move them to the cast
+	this->MoveWaitingListenersToCast();
 }
 
 
@@ -299,15 +298,12 @@ void Therver::HandleStopListeningCommandFromClient(ting::Ref<Client>& c){
 		return;
 	}
 
-	if(!c->isListening){
+	if(!c->IsListening() && !c->IsWaitingForListening()){
 		//already not listening
 		return;
 	}
 
-	ASSERT(c->listeningCast != this->casts.end())
-
-	c->isListening = false;
-	(*c->listeningCast).RemoveListener(c);
+	this->MoveClientToIdleState(c);
 }
 
 
@@ -319,30 +315,37 @@ void Therver::HandleListenNextCommandFromClient(ting::Ref<Client>& c){
 		return;
 	}
 
-	if(!c->isListening){
-		c->listeningCast = this->casts.begin();
-		c->isListening = true;
+	if(!c->IsListening()){
+		this->MoveClientToIdleState(c);
+
+		c->cast = this->casts.begin();
+		c->SetState(Client::LISTENING);
 	}else{
-		ASSERT(c->listeningCast != this->casts.end())
-		(*c->listeningCast).RemoveListener(c);
-		++c->listeningCast;
+		ASSERT(c->cast != this->casts.end())
+		(*c->cast).RemoveListener(c);
+		++c->cast;
 	}
 
+	ASSERT(c->IsListening())
+
 	//if we reached end try to jump to the beginning
-	if(c->listeningCast == this->casts.end()){
+	if(c->cast == this->casts.end()){
 		if(this->casts.size() > 0){
-			c->listeningCast = this->casts.begin();
-			ASSERT(c->listeningCast != this->casts.end())
+			c->cast = this->casts.begin();
+			ASSERT(c->cast != this->casts.end())
 		}else{
-			//there is no one to listen to
-			c->isListening = false;
-			c->listeningCast = this->casts.end();
+			//There is no one to listen to.
+			//Move to wait for listening state.
+			c->SetState(Client::WAITING_FOR_LISTENING);
+			this->waitingForListeningClients.push_back(c);
+			
 			c->SendStopListening();//send 'stop listening' to client
 			return;
 		}
 	}
 
-	(*c->listeningCast).AddListenerAndSendDataToClient(c);
+	ASSERT(c->cast != this->casts.end())
+	(*c->cast).AddListenerAndSendDataToClient(c);
 }
 
 
@@ -366,27 +369,29 @@ T_CastsIter Therver::RemoveCast(T_CastsIter castIter){
 		Cast& nextCast = *nextCastIter;
 
 		while(Ref<Client> c = cast.PopListener()){
-			ASSERT(c->isListening)
-			ASSERT(c->listeningCast == castIter)
+			ASSERT(c->IsListening())
+			ASSERT(c->cast == castIter)
 
-			c->listeningCast = nextCastIter;
+			c->cast = nextCastIter;
 			nextCast.AddListenerAndSendDataToClient(c);
 		}//~while
 	}else{//there is no one to listen to
 		while(Ref<Client> c = cast.PopListener()){
-			ASSERT(c->isListening)
-			ASSERT(c->listeningCast == castIter)
+			ASSERT(c->IsListening())
+			ASSERT(c->cast == castIter)
 
-			c->isListening = false;
-			c->listeningCast = this->casts.end();
+			//move to wait for listening state
+			c->SetState(Client::WAITING_FOR_LISTENING);
+			this->waitingForListeningClients.push_back(c);
+			
 			c->SendStopListening();//send 'stop listening' to client
 		}//~while
 	}
 
-	if(Ref<Client> plr = cast.GetPlayer()){
-		plr->isPlaying = false;
-		plr->playingCast = this->casts.end();
-	}
+	Ref<Client> plr = cast.GetPlayer();
+	ASSERT(plr)
+	plr->SetState(Client::IDLE);
+	plr->cast = this->casts.end();
 
 	return this->casts.erase(castIter);
 }
@@ -398,19 +403,19 @@ void UserPasswordQueriedMessage::Handle(){
 	ASSERT(this->c)
 	
 	if(this->res != SUCCESS){
-		this->c->SetState(Client::CONNECTED);
+		this->c->SetNetworkState(Client::CONNECTED);
 		this->c->SendLoginResult(false);//send login failure
 		return;
 	}
 
 	//compare passwords
 	if(this->c->password.compare(this->pw) != 0){
-		this->c->SetState(Client::CONNECTED);
+		this->c->SetNetworkState(Client::CONNECTED);
 		this->c->SendLoginResult(false);//send login failure
 		return;
 	}
 
-	this->c->SetState(Client::LOGGED_IN);
+	this->c->SetNetworkState(Client::LOGGED_IN);
 	this->c->SendLoginResult(true);//send login success
 }
 
@@ -457,3 +462,65 @@ void Therver::KillIdleCasts(){
 	this->killIdleCastsTimer.StartTimer();//schedule timer again
 }
 
+
+
+void Therver::MoveClientToIdleState(ting::Ref<Client>& c){
+	switch(c->GetState()){
+		case Client::IDLE:
+			//do nothing
+			return;
+			break;
+		case Client::PLAYING:
+			//if client has a playing cast then notify listeners that the cast has finished
+			//and remove the cast
+			this->RemoveCast(c->cast);
+			break;
+		case Client::LISTENING:
+			//remove the client from cast it listening to
+			ASSERT(c->cast != this->casts.end())
+			(*c->cast).RemoveListener(c);
+			c->cast = this->casts.end();
+			break;
+		case Client::WAITING_FOR_LISTENING:
+			this->RemoveClientFromWaitingListenersList(c);
+			break;
+		default:
+			ASSERT(false)
+			break;
+	}
+
+	c->SetState(Client::IDLE);
+}
+
+
+
+void Therver::MoveWaitingListenersToCast(){
+	ASSERT(this->casts.size() > 0)
+	while(this->waitingForListeningClients.size() > 0){
+		Ref<Client> c = this->waitingForListeningClients.back();
+		this->waitingForListeningClients.pop_back();
+
+		//add to cast
+		ASSERT(this->casts.size() > 0)
+		c->cast = this->casts.begin();
+		c->SetState(Client::LISTENING);
+		(*c->cast).AddListenerAndSendDataToClient(c);
+	}
+}
+
+
+
+void Therver::RemoveClientFromWaitingListenersList(ting::Ref<Client>& c){
+	for(T_ClientsIter i = this->waitingForListeningClients.begin();
+			i != this->waitingForListeningClients.end();
+			++i
+		)
+	{
+		if(*i == c){
+			this->waitingForListeningClients.erase(i);
+			return;
+		}
+	}
+	ASSERT(false)
+}
+
