@@ -276,18 +276,21 @@ void MusicActionMessage::Handle(){
 
 		switch(Preferences::Inst().GetTimbre()){
 			default:
-			case Preferences::SINE_PLUS_ADSR:
+			case SINE_PLUS_ADSR:
 				this->mt->musicActionsBuffer.push_back(0x00);
 				break;
-			case Preferences::PLUCKED_STRING:
+			case PLUCKED_STRING:
 				this->mt->musicActionsBuffer.push_back(0x01);
 				break;
-			case Preferences::TROMBONE:
+			case TROMBONE:
 				this->mt->musicActionsBuffer.push_back(0x02);
 				break;
-			case Preferences::CLARINET:
+			case CLARINET:
 				this->mt->musicActionsBuffer.push_back(0x03);
 				break;
+			case VIOLIN:
+				this->mt->musicActionsBuffer.push_back(0x04);
+				break;
 		}
 
 		//push 2 bytes for music actions counter
